'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from math import ceil
from pathlib import Path

from Scoring import ExaminedFile, FileScorer, Percentage


class BinaryScorer(FileScorer):
	"""
	Calculates a score based upon chunking the bytes of the files and comparing their contents.
	"""

	def __init__(self, chunk_size: int):
		"""
		Fully initialises the object.

		:raise ValueError: When the supplied chunk size is not greater than zero.

		:param chunk_size: The number of byte(s) per chunk to compare.
		"""
		FileScorer.__init__(self)

		if chunk_size <= 0:
			raise ValueError("The byte chunk size must be greater than zero.")
		self.__chunk_size = chunk_size

	@property
	def chunk_size(self) -> int:
		"""
		The number of byte(s) in a single chunk.
		"""
		return self.__chunk_size

	def score(self, source: ExaminedFile, compare: ExaminedFile) -> Percentage:
		"""
		Calculates the score of a file based upon its bytes.
		:param source: The source of the bytes whose score is to be calculated.
		:param compare: The bytes to compare against the source's bytes.
		:returns: The percentage of matching byte chunks.
		"""
		chunk_matches = 0
		source_size = Path(source.file_name_path()).stat().st_size
		compare_size = Path(compare.file_name_path()).stat().st_size

		if source_size < compare_size:
			small_file_name_path = source.file_name_path()
			large_file_name_path = compare.file_name_path()
			number_of_chunks = ceil(source_size / self.__chunk_size)
		else:
			small_file_name_path = compare.file_name_path()
			large_file_name_path = source.file_name_path()
			number_of_chunks = ceil(compare_size / self.__chunk_size)

		with open(small_file_name_path, 'rb') as small_file:
			with open(large_file_name_path, 'rb') as large_file:
				small_chunk = small_file.read(self.__chunk_size)
				large_chunk = large_file.read(self.__chunk_size)

				while small_chunk != b'' and large_chunk != b'':
					if small_chunk == large_chunk:
						chunk_matches += 1
						small_chunk = small_file.read(self.__chunk_size)
					large_chunk = large_file.read(self.__chunk_size)

		if number_of_chunks == 0:
			return Percentage(0)
		return Percentage(chunk_matches / number_of_chunks * 100.0)
