'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from os import path
from re import split

from Scoring import ExaminedFile, FileScorer, Percentage


class FileNameScorer(FileScorer):
	"""
	Calculates a score based upon the name of the files.
	"""

	__cache = {}


	def __init__(self):
		FileScorer.__init__(self)


	def score(self, source: ExaminedFile, compare: ExaminedFile) -> Percentage:
		"""
		Calculates the score of a file based upon its similarities of the words used in the name.
		:param source: The source of the file-name whose score is to be calculated.
		:param compare: The file-name to compare against the source's file-name.
		:returns: The percentage of similarities in words.
		"""

		# As scoring is only ever done once against a file, compare using the smallest name to give
		# the chance of the highest probable match, e.g.
		# source: Some-Long-Text-With-Lorem-Ipsum-In-It.txt and compare: Lorem-Ipsum.txt would score
		# significantly lower even though the entire name appears in the source.
		if len(source.file_name) < len(compare.file_name):
			small_words = FileNameScorer.__file_name_words(source)
			large_words = FileNameScorer.__file_name_words(compare)
		else:
			small_words = FileNameScorer.__file_name_words(compare)
			large_words = FileNameScorer.__file_name_words(source)

		matches = 0
		for small_word in small_words:
			for large_word in large_words:
				if small_word == large_word:
					matches += 1
		number_of_words = len(small_words)

		if number_of_words == 0:
			return Percentage(0)
		return Percentage(matches / number_of_words * 100.0)


	def __file_name_words(examine: ExaminedFile) -> set:
		if not examine.file_name in FileNameScorer.__cache:
			split_file_name_extension = path.splitext(examine.file_name)
			words = set(split('\W+', split_file_name_extension[0]))
			words.discard('')
			words.add(split_file_name_extension[1])
			words = set(word.upper() for word in words)
			FileNameScorer.__cache[examine.file_name] = words
		return FileNameScorer.__cache[examine.file_name]
