'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from abc import ABC, abstractmethod
from os import path, sep
from pathlib import Path
from progressbar import AdaptiveETA, Bar, Percentage as PercentageWidget, ProgressBar, \
	SimpleProgress, Timer
from typing import List


class ExaminedFile(object):
	"""
	Represents a file that is to be examined.
	"""

	def __init__(self, search_path: str, dir_path: str, file_name: str):
		"""
		Fully initialises the object.

		:raise ValueError: When the search path is not an absolute path.
		:raise ValueError: When the directory path is not an absolute path.
		:raise ValueError: When the directory path is not relative to the search path.
		:raise ValueError: When the file-name is empty.

		:param base_path: The absolute search path to the searching directory containing the examined
		file.
		:param dir_path: The absolute path to the directory that contains the file to examine.
		:param file_name: The name and extension of the file to examine.
		"""
		base = Path(search_path)

		if not base.is_absolute():
			raise ValueError("The directory search path '{}' must be absolute.".format(search_path))
		self.__search_path = str(base)

		if ('..' + sep) in dir_path or ('.' + sep) in dir_path:
			raise ValueError("The file directory path '{}' must be absolute.".format(dir_path))
		self.__dir_path = str(Path(dir_path).relative_to(self.__search_path))

		if len(file_name) == 0:
			raise ValueError('The file-name cannot be empty')
		self.__file_name = file_name

	@property
	def search_path(self) -> str:
		"""
		The absolute search path to the search directory.
		"""
		return self.__search_path

	@property
	def dir_path(self) -> str:
		"""
		The relative path to the directory that contains the file to examine from the search path.
		"""
		return self.__dir_path

	@property
	def file_name(self) -> str:
		"""
		The name and extension of the file to examine.
		"""
		return self.__file_name

	def file_name_path(self) -> str:
		"""
		Generates the absolute directory path and file-name with extension for the file being
		examined.
		"""
		return path.join(self.__search_path, self.relative_file_name_path())

	def relative_file_name_path(self) -> str:
		"""
		Generates the relative directory path and file-name with extension for the file being
		examined from the search path.
		"""
		return path.join(self.__dir_path, self.__file_name)

	def __str__(self):
		return self.file_name_path()


class Percentage(object):
	"""
	Represents a mathematical percentage.
	"""

	def __init__(self, value: int):
		"""
		Fully initialises the object.
		:raises ValueError: When the supplied percentage value is less than zero.
		:raises ValueError: When the supplied percentage value is greater than one-hundred.
		:param value: The percentage value whole number.
		"""
		if value < 0:
			raise ValueError("The percentage value, {}, cannot be below zero.".format(value))

		if value > 100:
			raise ValueError("The percentage value, {}, cannot be above one-hundred.".format(value))
		self.__value = int(value)

	@property
	def value(self) -> int:
		"""
		The percentage value represented.
		"""
		return self.__value

	def __lt__(self, other):
		return self.__value < other.__value

	def __gt__(self, other):
		return self.__value > other.__value

	def __eq__(self, other):
		return self.__value == other.__value

	def __le__(self, other):
		return self.__value <= other.__value

	def __ge__(self, other):
		return self.__value >= other.__value

	def __str__(self):
		return "{}%".format(self.__value)


class FileScorer(ABC):
	"""
	An object that produces a score for a file.
	"""

	def __init__(self):
		ABC.__init__(self)

	@abstractmethod
	def score(self, source: ExaminedFile, compare: ExaminedFile) -> Percentage:
		"""
		When implemented generates a percentage score based on the similarities of the two files
		using the implemented criteria.
		:param source: The source of the file to be scored.
		:param compare: The file to compare against the source being scored.
		:returns: The similarity percentage of the two files.
		"""
		pass


class ScoredFile(object):
	"""
	Represents a file that is scored.
	"""

	def __init__(self, source: ExaminedFile, compare: ExaminedFile, percent: Percentage):
		self.__source = source
		self.__compare = compare
		self.__percent = percent

	@property
	def source(self) -> ExaminedFile:
		"""
		The file that was scored.
		"""
		return self.__source

	@property
	def compare(self) -> ExaminedFile:
		"""
		The file that was compared during the scoring.
		"""
		return self.__compare

	@property
	def percent(self) -> Percentage:
		"""
		The percentage value of how similar the two files are.
		"""
		return self.__percent

	def __str__(self):
		return "'{}' to '{}' scores {}.".format(self.__source, self.__compare, self.__percent)


def score_files(files: List[ExaminedFile], scorer: FileScorer, score_threshold: Percentage):
	"""
	Produces a similarity score for every supplied file.
	:param files: The files whose scores should be generated for similarity.
	:param scorer: The method used to score files for similarity.
	:param score_threshold: The level at or above that a similar file should be reported.
	"""
	size = len(files)

	with ProgressBar(prefix='Scoring Progress:', max_value=size, redirect_stdout=True, widgets=[
		' ', PercentageWidget(), ' ', SimpleProgress(format='(%(value_s).2f of %(max_value_s)s)'),
		' ', Bar(), ' ', Timer(), ' / ', AdaptiveETA()]) as progress:
		for source_index, source in enumerate(files):
			scores = []
			# Add one to skip comparing source against itself.
			compare_index = source_index + 1

			while compare_index < size:
				compare = files[compare_index]
				scores.append(ScoredFile(source, compare, scorer.score(source, compare)))
				progress.update(source_index + ((compare_index + 1.0) / size))
				compare_index += 1
			print("{}\t".format(source.relative_file_name_path()), end='')

			is_first_print = True
			for score in scores:
				if score.percent >= score_threshold:
					format_string = "{}\v{}".format(score.compare.relative_file_name_path(),
										score.percent)

					if is_first_print:
						print(format_string, end='')
						is_first_print = False
					else:
						print("\t" + format_string, end='')
			print('')
			progress.update(source_index + 1.0)
