#!/usr/bin/env python3
'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from argparse import ArgumentParser
from os import getcwd, walk
from sys import stderr
from traceback import format_exc

import __init__ as Info
from BinaryScoring import BinaryScorer
from NameScoring import FileNameScorer
from Scoring import ExaminedFile, Percentage, score_files


def __process_arguments():
	parser = ArgumentParser(description=Info.DESCRIPTION)
	parser.add_argument('-s', '--search', action='store', type=str, default=getcwd(),
					 help='The directory path to search for similar files. Default current '
					 'working directory.')
	parser.add_argument('-t', '--threshold', action='store', type=int, default=20,
					 help='The percentage threshold, at or above, to report a similarity between '
					 'files. Default 20%%.')
	parser.add_argument('-m', '--method', action='store', default='name', type=str,
					 choices=['n', 'name', 'b', 'binary'],
					 help='Methods for scoring similarity between files. Default name.')

	#{ Binary Score Method Options

	binary_group = parser.add_argument_group('Binary Method',
										  'The options for configuring the binary scoring method.')
	binary_group.add_argument('-c', '--chunk-size', action='store', default=64,
								 type=int, help='The number of bytes (chunk) to process at a '
								 'time. Default 64 bytes.')
	parser.add_argument_group(binary_group)

	#}

	return parser.parse_args()


def __selected_scorer_method(method: str, args):
	if ('name' ==  method or 'n' == method):
		return FileNameScorer()
	elif ('binary' ==  method or 'b' == method):
		return BinaryScorer(args.chunk_size)
	else:
		print("Unknown scorer method supplied '{}'.".format(method), file=stderr)
		exit(1)


if __name__ == "__main__":
	try:
		print("""{long_name} {version} Copyright (C) {years} John Schneiderman
{short_name} comes with ABSOLUTELY NO WARRANTY;
This is free software, and you are welcome to redistribute it
under certain conditions; see the COPYING file for details,
or the Free Software Foundation's GPL.
""".format(long_name=Info.LONG_NAME, version=Info.VERSION, years=Info.YEARS,
		   short_name=Info.SHORT_NAME),
			file=stderr
		)
		args = __process_arguments()
		files = []

		for (dir_path, dir_names, file_names) in walk(args.search):
			for file_name in file_names:
				files.append(ExaminedFile(args.search, dir_path, file_name))
		scorer_method = __selected_scorer_method(args.method, args)
		score_files(files, scorer_method, Percentage(args.threshold))
		exit(0)
	except (ValueError, RuntimeError) as error:
		print("Execution Failed: {}".format(error), file=stderr)
		exit(2)
	except Exception as error:
		print("Unexpected error: {err}\n{banner}\n{trace}{banner}".format(err=error,
																		banner='-' * 60,
																		trace=format_exc()),
		file=stderr)
		exit(3)
