'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from os import path
import unittest

import __init__ as Paths
import AddSimilisPath
from Scoring import ExaminedFile, Percentage
from BinaryScoring import BinaryScorer


class BinaryScorerTests(unittest.TestCase):
	def setUp(self):
		self.__scorer = BinaryScorer(1)

	def test_constructor_should_have_all_members_set(self):
		self.assertEqual(self.__scorer.chunk_size, 1)

	def test_constructor_value_less_than_minimum_should_throw(self):
		with self.assertRaises(ValueError):
			percentage = BinaryScorer(0)

	def test_all_chunks_in_common_should_be_100_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		score = self.__scorer.score(source, compare)
		self.assertEqual(score, Percentage(100))

	def test_all_chunks_in_common_leading_and_trialing_noise_should_be_100_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet-numbers.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		score = self.__scorer.score(source, compare)
		self.assertEqual(score, Percentage(100))

	def test_all_chunks_in_common_separated_by_noise_should_be_100_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'numbers-spaced.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'numbers.txt')
		score = self.__scorer.score(source, compare)
		self.assertEqual(score, Percentage(100))

	def test_half_chunks_in_common_should_be_46_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet-symbol.txt')
		score = self.__scorer.score(source, compare)
		self.assertEqual(score, Percentage(46))

	def test_with_chunk_size_2_should_be_42_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet-symbol.txt')
		score = BinaryScorer(2).score(source, compare)
		# It's lower now since the last chunk containing a letter does not match any later chunks.
		self.assertEqual(score, Percentage(42))

	def test_no_chunks_in_common_should_be_0_percent(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'numbers.txt')
		score = self.__scorer.score(source, compare)
		self.assertEqual(score, Percentage(0))

	def test_chunk_size_larger_than_file_should_process(self):
		source = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		compare = ExaminedFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		score = BinaryScorer(128).score(source, compare)
		self.assertEqual(score, Percentage(100))


if __name__ == "__main__":
	unittest.main()
