'''*****************************************************************************
***  This file is part of Similis.                                           ***
***                                                                          ***
***  Copyright (C) 2020                                                      ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU General Public License as published          ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU General Public License for more details.                    ***
***                                                                          ***
***  You should have received a copy of the GNU General Public License       ***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''
from os import path
import unittest

import __init__ as Paths
import AddSimilisPath
from Scoring import ExaminedFile, Percentage, ScoredFile, score_files
from Testers import TestFile, TestFileScorer


def suite():
	suite = unittest.TestSuite()
	suite.addTest(ExaminedFileTests())
	suite.addTest(PercentageTests())
	suite.addTest(ScoredFileTests())


class ExaminedFileTests(unittest.TestCase):
	search_path = '/usr/games'
	game_file_path = path.join(search_path, 'game', 'subdirectory')
	expected_dir_parth = path.join('game', 'subdirectory')
	file_name = 'Lorem-Ipsum.txt'

	def test_constructor_should_have_all_members_set(self):
		examine = ExaminedFile(ExaminedFileTests.search_path, ExaminedFileTests.game_file_path,
						 ExaminedFileTests.file_name)
		self.assertEqual(examine.search_path, ExaminedFileTests.search_path)
		self.assertEqual(examine.dir_path, ExaminedFileTests.expected_dir_parth)
		self.assertEqual(examine.file_name, ExaminedFileTests.file_name)

	def test_constructor_relative_search_path_should_throw(self):
		with self.assertRaises(ValueError):
			ExaminedFile('../games', '../games', ExaminedFileTests.file_name)

	def test_constructor_no_common_dir_path_should_throw(self):
		with self.assertRaises(ValueError):
			ExaminedFile(ExaminedFileTests.search_path, 'some/dir', ExaminedFileTests.file_name)

	def test_constructor_relative_dir_path_should_throw(self):
		with self.assertRaises(ValueError):
			ExaminedFile(ExaminedFileTests.search_path,
				path.join(ExaminedFileTests.search_path, '..', 'some', 'dir'),
				ExaminedFileTests.file_name)

	def test_constructor_empty_file_name_should_throw(self):
		with self.assertRaises(ValueError):
			ExaminedFile(ExaminedFileTests.search_path, ExaminedFileTests.game_file_path, '')

	def test_file_name_path_should_be_absolute_path(self):
		examine = ExaminedFile(ExaminedFileTests.search_path, ExaminedFileTests.game_file_path,
						 ExaminedFileTests.file_name)
		self.assertEqual(
			examine.file_name_path(),
			path.join(ExaminedFileTests.game_file_path, ExaminedFileTests.file_name)
		)

	def test_relative_file_name_path_should_be_supplied_relative_path(self):
		examine = ExaminedFile(ExaminedFileTests.search_path, ExaminedFileTests.game_file_path,
						 ExaminedFileTests.file_name)
		self.assertEqual(
			examine.relative_file_name_path(),
			path.join(ExaminedFileTests.expected_dir_parth, ExaminedFileTests.file_name)
		)


class PercentageTests(unittest.TestCase):
	def test_constructor_should_have_all_members_set(self):
		percentage = Percentage(57)
		self.assertEqual(percentage.value, 57)

	def test_constructor_value_at_minimum_should_be_set(self):
		percentage = Percentage(0)
		self.assertEqual(percentage.value, 0)

	def test_constructor_value_less_than_minimum_should_throw(self):
		with self.assertRaises(ValueError):
			percentage = Percentage(-1)

	def test_constructor_value_at_maximum_should_be_set(self):
		percentage = Percentage(100)
		self.assertEqual(percentage.value, 100)

	def test_constructor_value_greater_than_maximum_should_throw(self):
		with self.assertRaises(ValueError):
			percentage = Percentage(101)

	def test_relational_operators_should_be_true(self):
		five = Percentage(5)
		four = Percentage(4)
		self.assertTrue(five == five)
		self.assertTrue(four != five)
		self.assertTrue(four < five)
		self.assertTrue(four <= four)
		self.assertTrue(five > four)
		self.assertTrue(four >= four)


class ScoredFileTests(unittest.TestCase):
	def test_constructor_should_have_all_members_set(self):
		source = ExaminedFile('/files', '/files/dir0', 'Lorem-Ipsum.txt')
		compare = ExaminedFile('/files', '/files/dir1', 'Text-With-Lorem-Ipsum-In-It.txt')
		percentage = Percentage(57)
		file = ScoredFile(source, compare, percentage)
		self.assertEqual(file.source, source)
		self.assertEqual(file.compare, compare)
		self.assertEqual(file.percent, percentage)


class ScoreFilesTests(unittest.TestCase):
	def test_score_files_should_only_source_files_once(self):
		tf0 = TestFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet-numbers.txt')
		tf1 = TestFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet-symbol.txt')
		tf2 = TestFile(Paths.test_dir_path, Paths.test_data_dir_path, 'alphabet.txt')
		tf3 = TestFile(Paths.test_dir_path, Paths.test_data_dir_path, 'numbers-spaced.txt')
		tf4 = TestFile(Paths.test_dir_path, Paths.test_data_dir_path, 'numbers.txt')
		score_files([tf0, tf1, tf2, tf3, tf4], TestFileScorer(), Percentage(100))

		self.assertEqual(tf0.times_as_source, 4)
		self.assertEqual(tf0.times_as_compare, 0)

		self.assertEqual(tf1.times_as_source, 3)
		self.assertEqual(tf1.times_as_compare, 1)

		self.assertEqual(tf2.times_as_source, 2)
		self.assertEqual(tf2.times_as_compare, 2)

		self.assertEqual(tf3.times_as_source, 1)
		self.assertEqual(tf3.times_as_compare, 3)

		self.assertEqual(tf4.times_as_source, 0)
		self.assertEqual(tf4.times_as_compare, 4)


if __name__ == "__main__":
	runner = unittest.TextTestRunner()
	runner.run(suite())
